import type { ComputedRef, Reactive, Ref } from 'vue'
import type {
  CustomFilter as CustomFilterInfo,
  DatePickerCustomFilter,
  FilterItem,
  InputCustomFilter,
  SelectCustomFilter
} from '@/types/customFilter.ts'
import type { QueryInfo } from '@/types/table.ts'

import { nextTick, reactive, ref } from 'vue'
import { CustomFilterValueType } from '@/types/customFilter.ts'

import CustomFilter from '@/components/form/CustomFilter.vue'

type CustomFilterRef = InstanceType<typeof CustomFilter>

export function useCustomFilter(customFilterRef: Ref<CustomFilterRef | undefined>) {
  // 自定义筛选对话框开关
  const customFilterDialog = ref<boolean>(false)

  // 筛选的信息，主要保存类型等
  const customFilterInfo = reactive<Record<string, CustomFilterInfo>>({})

  // 筛选信息的数据，存真正筛选的值
  const customFilterList = reactive<Record<string, Array<FilterItem>>>({})

  // 当前激活的筛选框是哪个
  const activeCustomFilterKey = ref('')

  /**
   * 生成筛选信息
   *
   * 根据customFilterValueType 生成不同的模板
   *
   * @param item 筛选项
   */
  const createCustomFilterItem = (item: QueryInfo) => {
    if (item.otherOption.customFilterValueType === CustomFilterValueType.DATEPICKER) {
      return {
        valueType: CustomFilterValueType.DATEPICKER,

        format: item.otherOption.format,
        valueFormat: item.otherOption.valueFormat
      } as DatePickerCustomFilter
    }
    if (item.otherOption.customFilterValueType === CustomFilterValueType.INPUT) {
      return {
        valueType: CustomFilterValueType.INPUT,
        placeholder: item.otherOption.placeholder,
        valueValid: item.otherOption.valueValid,
        valueFormat: item.otherOption.valueFormat
      } as InputCustomFilter
    }
    if (item.otherOption.customFilterValueType === CustomFilterValueType.SELECT) {
      return {
        valueType: CustomFilterValueType.SELECT,
        options: item.otherOption.options
      } as SelectCustomFilter
    }
    return false
  }

  /**
   * 初始化自定义筛选器的信息
   *
   * 为每个筛选器的customFilterList给一个初始值，方便用户打开的时候有至少一个选项
   *
   * 初始化每个筛选器的信息，为筛选器赋值类型，让筛选器根据类型不同，渲染不同组件。
   *
   */
  const initCustomFilterInfo = (customFieldsList: ComputedRef<QueryInfo[] | undefined>) => {
    customFieldsList.value?.forEach((item) => {
      customFilterList[item.name] = [
        {
          filterCondition: 'gt',
          value: '' // 这个值无所谓，因为后面会根据类型来渲染不同的组件
        }
      ]

      let customFilterItem = createCustomFilterItem(item)
      if (customFilterItem !== false) {
        customFilterInfo[item.name] = customFilterItem
      }
    })
  }

  /**
   * 打开自定义筛选为筛选器赋初值
   *
   * @param item 当前字段信息
   */
  const openCustomFilter = (item: QueryInfo) => {
    activeCustomFilterKey.value = item.name
    customFilterDialog.value = true
    // 这里一定要在下一个tick中执行，因为组件还没渲染出来，所以无法获取到dom元素
    void nextTick(() => {
      if (customFilterRef.value) {
        customFilterRef.value.initFilterList()
      }
    })
  }

  /**
   * 将自定义过滤器列表转换为查询参数。
   * @param filterList 过滤器值列表
   */
  const conversionCustomFilter = (
    filterList: Array<{
      filterCondition: string
      value: any
    }>
  ) => {
    let result: Record<string, any> = {}
    filterList.forEach((item) => {
      result[item.filterCondition] = item.value
    })
    return result
  }

  /**
   * 提交筛选条件
   *
   * 提交前需要验证数据是否符合格式，不符合则不提交。
   *
   */
  const confirmCustomFilter = (queryFormData: Reactive<Record<string, any>>) => {
    if (customFilterRef.value && customFilterRef.value.verifySub()) {
      customFilterDialog.value = false
      let result = customFilterRef.value.subCondition()
      // 拿到数据后，需要转为查询参数
      queryFormData[activeCustomFilterKey.value] = conversionCustomFilter(result)
      // 保存筛选值
      customFilterList[activeCustomFilterKey.value] = result
    }
  }

  return {
    customFilterDialog,
    customFilterInfo,
    customFilterList,
    activeCustomFilterKey,

    initCustomFilterInfo,
    openCustomFilter,

    confirmCustomFilter
  }
}
